package com.compo.camera.dialog;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.DialogFragment;
import android.content.res.Resources;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.GradientDrawable;
import android.os.Bundle;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.BaseAdapter;
import android.widget.GridView;
import android.widget.ImageView;

import com.compo.inspector.R;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by lmodlinski on 25/03/17.
 */
public class ColorPickerDialog extends DialogFragment
{
	private ColorGridAdapter adapter;
	private GridView grid;

	private int[] colors = {};
	private int value = 0;
	private int columns = 3;

	private int layoutId = R.layout.color_grid;

	private int selected;

	protected static final String KEY_TITLE_ID = "title_id";
	protected static final String KEY_COLORS = "colors";
	protected static final String KEY_SELECTED_COLOR = "selected_color";
	protected static final String KEY_COLUMNS = "columns";

	private OnColorSelectedListener listener;

	public ColorPickerDialog()
	{

	}

	public static ColorPickerDialog getInstance()
	{
		return new ColorPickerDialog();
	}

	public static ColorPickerDialog getInstance(int[] colors, int color, int columns)
	{
		ColorPickerDialog picker = ColorPickerDialog.getInstance();
		picker.initialize(colors, color, columns);

		return picker;
	}

	public void setArguments(int columns)
	{
		Bundle bundle = new Bundle();
		bundle.putInt(KEY_COLUMNS, columns);

		setArguments(bundle);
	}

	public void initialize(int[] colors, int color, int columns)
	{
		this.colors = colors;
		this.columns = columns;
		this.selected = color;

		this.setArguments(this.columns);
	}

	/**
	 * Interface for a callback when a color square is selected.
	 */
	public interface OnColorSelectedListener
	{

		/**
		 * Called when a specific color square has been selected.
		 */
		public void onColorSelected(int color);
	}


	public void setOnColorSelectedListener(OnColorSelectedListener listener)
	{
		this.listener = listener;
	}

	public void setPreference()
	{
		tryBindLists();
	}

	@Override
	public void onStart()
	{
		super.onStart();

		if (null != this.getDialog()) {
			this.getDialog().getWindow().setLayout(1000, 730);
			this.getDialog().getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
		}
	}

	@Override
	public void onAttach(Activity activity)
	{
		super.onAttach(activity);

		if (0 < this.colors.length) {
			tryBindLists();
		}
	}

	@Override
	public Dialog onCreateDialog(Bundle savedInstanceState)
	{
		LayoutInflater layoutInflater = LayoutInflater.from(getActivity());
		View rootView = layoutInflater.inflate(R.layout.color_dialog, null);

		if (this.getArguments() != null) {
			this.columns = this.getArguments().getInt(KEY_COLUMNS);
		}

		if (savedInstanceState != null) {
			this.colors = savedInstanceState.getIntArray(KEY_COLORS);
			this.selected = (Integer) savedInstanceState.getSerializable(KEY_SELECTED_COLOR);

			this.tryBindLists();
		}

		this.grid = (GridView) rootView.findViewById(R.id.color_dialog_id);
		this.grid.setNumColumns(this.columns);
		this.grid.setOnItemClickListener(new AdapterView.OnItemClickListener()
		{
			@Override
			public void onItemClick(AdapterView<?> listView, View view,
									int position, long itemId)
			{
				if (listener != null) {
					listener.onColorSelected(adapter.getItem(position));
				}

				dismiss();
			}
		});

		tryBindLists();

		return new AlertDialog.Builder(getActivity())
				.setView(rootView)
				.create();
	}

	private void tryBindLists()
	{
		if (isAdded() && this.adapter == null) {
			this.adapter = new ColorGridAdapter();
		}

		if (this.adapter != null && this.grid != null) {
			this.adapter.setSelectedColor(this.selected);  //USE this to select color
			this.grid.setAdapter(this.adapter);
		}
	}

	@Override
	public void onSaveInstanceState(Bundle outState)
	{
		super.onSaveInstanceState(outState);

		outState.putIntArray(KEY_COLORS, this.colors);
		outState.putSerializable(KEY_SELECTED_COLOR, this.selected);
	}


	private static void setColorViewValue(View view, int color)
	{
		if (view instanceof ImageView) {
			ImageView imageView = (ImageView) view;
			Resources res = imageView.getContext().getResources();

			Drawable currentDrawable = imageView.getDrawable();
			GradientDrawable colorChoiceDrawable;
			if (currentDrawable != null && currentDrawable instanceof GradientDrawable) {
				// Reuse drawable
				colorChoiceDrawable = (GradientDrawable) currentDrawable;
			} else {
				colorChoiceDrawable = new GradientDrawable();
				colorChoiceDrawable.setShape(GradientDrawable.OVAL);
			}

			// Set stroke to dark version of color
			int darkenedColor = Color.rgb(
					Color.red(color) * 192 / 256,
					Color.green(color) * 192 / 256,
					Color.blue(color) * 192 / 256);

			colorChoiceDrawable.setColor(color);
			colorChoiceDrawable.setStroke((int) TypedValue.applyDimension(
					TypedValue.COMPLEX_UNIT_DIP, 1, res.getDisplayMetrics()), darkenedColor);

			imageView.setImageDrawable(colorChoiceDrawable);

		}
	}

	private class ColorGridAdapter extends BaseAdapter
	{
		private List<Integer> choice = new ArrayList<Integer>();
		private int selected;

		private ColorGridAdapter()
		{
			for (int color : colors) {
				this.choice.add(color);
			}
		}

		@Override
		public int getCount()
		{
			return this.choice.size();
		}

		@Override
		public Integer getItem(int position)
		{
			return this.choice.get(position);
		}

		@Override
		public long getItemId(int position)
		{
			return this.choice.get(position);
		}

		@Override
		public View getView(int position, View convertView, ViewGroup container)
		{
			if (convertView == null) {
				convertView = LayoutInflater.from(getActivity()).inflate(layoutId, container, false);
			}

			int color = getItem(position);
			setColorViewValue((ImageView) convertView.findViewById(R.id.color_view), color);
			setBorderView((ImageView) convertView.findViewById(R.id.color_border), Color.WHITE);
			setBorderView((ImageView) convertView.findViewById(R.id.color_frame), Color.rgb(136, 23, 5));

			convertView.findViewById(R.id.color_selected).setVisibility(color == selected ? View.VISIBLE : View.INVISIBLE);

			return convertView;
		}

		private void setBorderView(ImageView border, int color)
		{
			Drawable currentDrawable = border.getDrawable();
			GradientDrawable colorChoiceDrawable;
			if (currentDrawable != null && currentDrawable instanceof GradientDrawable) {
				// Reuse drawable
				colorChoiceDrawable = (GradientDrawable) currentDrawable;
			} else {
				colorChoiceDrawable = new GradientDrawable();
				colorChoiceDrawable.setShape(GradientDrawable.OVAL);
			}

			Resources res = border.getContext().getResources();

			// Set stroke to dark version of color
			colorChoiceDrawable.setColor(color);
			colorChoiceDrawable.setStroke((int) TypedValue.applyDimension(
					TypedValue.COMPLEX_UNIT_DIP, 1, res.getDisplayMetrics()), color);

			border.setImageDrawable(colorChoiceDrawable);
		}

		public void setSelectedColor(int selectedColor)
		{
			if (selected != selectedColor) {
				selected = selectedColor;
				notifyDataSetChanged();
			}
		}
	}
}
